from pyhap import loader


class ServiceBase(type):
    def __new__(cls, name, bases, attrs):
        new_class = super().__new__(cls, name, bases, attrs)

        parents = [b for b in bases if isinstance(b, ServiceBase)]
        if not parents:
            return new_class

        if isinstance(new_class.service, str):
            new_class.service = loader.get_serv_loader().get(new_class.service)

        return new_class


class Service(object, metaclass=ServiceBase):
    def run(self, sentinel):
        pass

    def stop(self):
        pass



from pathlib import Path
import subprocess
import random

from .base import Service


class TemperatureSensor(Service):
    service = 'TemperatureSensor'

    def __init__(self, *args, **kwargs):
        self.refresh_time = kwargs.pop('refresh', 10)
        self.current_temperature = self.service.get_characteristic('CurrentTemperature')
        super().__init__(*args, **kwargs)

    def update_temperature(self):
        current_temperature = self.read_current_temperature()
        if current_temperature is not None:
            self.current_temperature.set_value(current_temperature)

    def run(self, sentinel):
        self.update_temperature()
        while not sentinel.wait(self.refresh_time):
            self.update_temperature()


class FakeTemperature(TemperatureSensor):
    def __init__(self, *args, **kwargs):
        self.minimum_value = kwargs.pop('minimum', 0)
        self.maximum_value = kwargs.pop('maximum', 40)
        super().__init__(*args, **kwargs)

    def read_current_temperature(self):
        return random.randint(self.minimum_value, self.maximum_value)


class DS18B20(TemperatureSensor):
    def __init__(self, *args, **kwargs):
        self.sensor_id = kwargs.pop('sensor_id')
        super().__init__(*args, **kwargs)

    @property
    def temperature_file(self):
        return Path('/sys/bus/w1/devices/') / self.sensor_id / 'w1_slave'

    @property
    def temperature_raw_data(self):
        return self.temperature_file.open().read()

    def read_current_temperature(self):
        raw_data = self.temperature_raw_data.split('\n')
        if raw_data[0].strip().split()[-1] == 'YES':
            return float(raw_data[1].strip().split('t=')[1]) / 1000


class RemoteDS18B20(DS18B20):
    def __init__(self, *args, **kwargs):
        self.server = kwargs.pop('server')
        super().__init__(*args, **kwargs)

    @property
    def temperature_raw_data(self):
        return subprocess.check_output([
            'ssh',
            self.server,
            'cat',
            str(super().temperature_file)
        ]).decode('utf-8')



import logging
import signal
import threading
import random

from collections import OrderedDict
import pyhap.accessory

from .services.base import Service


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__file__)


class AccessoryBase(type):

    def __new__(cls, name, bases, attrs):
        new_class = super().__new__(cls, name, bases, attrs)

        parents = [b for b in bases if isinstance(b, AccessoryBase)]
        if not parents:
            return new_class

        if isinstance(new_class.category, str):
            new_class.category = getattr(pyhap.accessory.Category, attrs['category'])

        new_class._services = OrderedDict()

        for name, attr in attrs.items():
            if isinstance(attr, Service):
                new_class._services[name] = attr

        if not hasattr(new_class, 'AccessoryInformation'):
            new_class.AccessoryInformation = type('AccessoryInformation', (object,), {})

        return new_class


class Accessory(pyhap.accessory.Accessory, metaclass=AccessoryBase):
    def __init__(self, *args, **kwargs):
        self.persist_file = kwargs.pop('persist_file', 'accessory.state')
        self.port = kwargs.pop('port', random.randint(50000, 60000))
        if not args:
            args = ('Name', )
        super().__init__(*args, **kwargs)

    @property
    def driver(self):
        if not hasattr(self, '_driver'):
            from pyhap.accessory_driver import AccessoryDriver
            self._driver = AccessoryDriver(self, port=self.port, persist_file=self.persist_file)
            signal.signal(signal.SIGINT, self._driver.signal_handler)
            signal.signal(signal.SIGTERM, self._driver.signal_handler)
        return self._driver

    def start(self):
        self.driver.start()

    def get_accessory_information(self):
        info_service = pyhap.loader.get_serv_loader().get('AccessoryInformation')

        for attr in ['Name', 'Manufacturer', 'Model', 'SerialNumber']:
            info_service.get_characteristic(attr).set_value(
                getattr(self.AccessoryInformation, attr, attr)
            )

        if self.display_name is not None:
            info_service.get_characteristic('Name').set_value(
                self.display_name
            )

        return info_service

    def _set_services(self):
        # We no longer need to define _set_services(), as we define this using attributes.
        self.add_service(self.get_accessory_information())
        for name, service in self._services.items():
            if service.service not in self.services:
                self.add_service(service.service)

    def run(self):
        super().run()
        for name, service in self._services.items():
            threading.Thread(target=service.run, name=name, args=[self.run_sentinel]).start()

    def stop(self):
        super().stop()
        for name, service in self._services.items():
            service.stop()



import logging
import time

from .base import Service


OPEN = 0        # Valid for target and current states
CLOSED = 1      # Valid for target and current states
OPENING = 2     # Valid only for current state
CLOSING = 3     # Valid only for current state
STOPPED = 4     # Valid only for current state

logger = logging.getLogger(__file__)


class GarageDoor(Service):
    service = 'GarageDoorOpener'

    def __init__(self, *args, **kwargs):
        self.setup_gpio(**kwargs.pop('gpio_pins'))
        self.current_state = self.service.get_characteristic('CurrentDoorState')
        self.target_state = self.service.get_characteristic('TargetDoorState')
        self.target_state.setter_callback = self.set_target
        super().__init__(*args, **kwargs)

    def set_target(self, value):
        """
        Defines the behaviour that should happen when a HomeKit trigger event
        is sent. This will have already set the target_state to the opposite to
        what it was before (ie, if the door was already open, or opening,
        then the target_state will now be closed). Keep in mind that target_state
        only has OPEN/CLOSED, so HomeKit is not able to model "stopped in the middle"
        very well.
        """

    def setup_gpio(self, **pins):
        """
        Override the things that need to happen when the system starts up, and
        pins need configuring.
        """

    def door_is_open(self):
        logger.info("Open")
        self.current_state.set_value(OPEN)
        self.target_state.value = OPEN
        self.target_state.notify()

    def door_is_opening(self):
        logger.info("Opening")
        self.current_state.set_value(OPENING)
        self.target_state.value = OPEN
        self.target_state.notify()
        # self.start_stopped_timer()

    def door_is_closed(self):
        logger.info("Closed")
        self.current_state.set_value(CLOSED)
        self.target_state.value = CLOSED
        self.target_state.notify()

    def door_is_closing(self):
        logger.info("Closing")
        self.current_state.set_value(CLOSING)
        self.target_state.value = CLOSED
        self.target_state.notify()
        # self.start_stopped_timer()


class SingleButtonTwoSensorGarageDoor(GarageDoor):
    def setup_gpio(self, **pins):
        from gpiozero import Button, LED
        self.relay = LED(pins['relay'])
        self.top_limit = Button(pins['top_limit'])
        self.bottom_limit = Button(pins['bottom_limit'])

    def run(self, sentinel):
        """
        This uses gpiozero, which means we can use the really nice event
        handlers it provides. We then just need to detect what the current
        state is at startup, and set that in HomeKit.
        """
        self.top_limit.when_pressed = self.door_is_open
        self.top_limit.when_released = self.door_is_closing
        self.bottom_limit.when_pressed = self.door_is_closed
        self.bottom_limit.when_released = self.door_is_opening

        # Set initial state.
        if self.top_limit.is_pressed:
            self.door_is_open()
        elif self.bottom_limit.is_pressed:
            self.door_is_closed()

    def set_target(self, value):
        """
        This garage door system has a single button that triggers different
        behaviour depending upon the current state of the system. This mostly
        matches with the behaviour from HomeKit, with one exception: when the
        door is currently opening, a single press halts the door,
        and then a second press reverses the direction.

        The HomeKit assumption is that a single press reverses the direction,
        so we need to stop, wait for a second, and then re-trigger.

        However, if we are being set from HomeKit, then we know the target state,
        so we can be cleverer than the raw remote, and not trigger anything if
        we are already in that state (or heading towards it).
        """
        if self.current_state.value == self.target_state.value:
            return
        if self.current_state.value == OPENING:
            if self.target_state.value == OPEN:
                return
            self.trigger_button()
            time.sleep(1)
        if self.current_state.value == CLOSING and self.target_state.value == CLOSED:
            return
        self.trigger_button()

    def trigger_button(self):
        logger.info('Button pressed')
        self.relay.on()
        time.sleep(1)
        self.relay.off()
        logger.info('Button released')



from setuptools import setup, find_packages
from pathlib import Path


with (Path(__file__).parent / 'README.rst').open() as fp:
    long_description = fp.read()


setup(
    name='gleeful',
    version='0.1.1',
    description='HomeKit Accessory Protocol (HAP) wrapper for Python',
    long_description=long_description,
    url='https://bitbucket.org/schinckel/gleeful/',
    author='Matthew Schinckel',
    author_email='matt@schinckel.net',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python',
        'Topic :: Home Automation',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=find_packages(exclude=['docs', 'tests']),
    install_requires=[
        'hap-python'
    ],
)